use std::{fs, path::PathBuf, str::FromStr};

use log::info;
use ropey::RopeSlice;
use tower_lsp::lsp_types::{CompletionItem, CompletionItemKind, CompletionList, Position, Url};

use crate::utils::{resolve_path, to_escape_path};

pub fn include_path_completion(uri: &Url, line: &RopeSlice, pos: Position) -> Option<CompletionList> {
    let line_text = line.as_str().unwrap_or("");
    if line_text.trim().starts_with("`include") {
        let character = pos.character as usize;
        let first_quote_idx = line_text.find("\"");
        let last_quote_idx = line_text.rfind("\"");
        
        if first_quote_idx.is_none() || last_quote_idx.is_none() {
            return None;
        }
        let first_quote_idx = first_quote_idx.unwrap();
        let last_quote_idx = last_quote_idx.unwrap();

        if character >= first_quote_idx && character <= last_quote_idx {
            let mut path_string = &line_text[(first_quote_idx + 1) .. last_quote_idx];
            if path_string.starts_with("./") || path_string.starts_with(".\\") {
                path_string = &path_string[2 ..];
            }
            
            // 路径转换
            let path = match PathBuf::from_str(uri.path()) {
                Ok(path) => path,
                Err(error) => {
                    info!("error happen in <goto_include_definition>: {:?}", error);
                    return None;
                }
            };
            let escape_path = to_escape_path(&path);
            let escape_path = escape_path.to_str().unwrap_or("");
            if escape_path.len() == 0 {
                return None;
            }

            if let Some(abs_path) = resolve_path(escape_path, path_string) {                
                // 遍历该路径下所有文件和文件夹，并构建新的自动补全列表
                info!("completion active in {:?}", abs_path);
                if abs_path.is_dir() {
                    let mut completion_items = Vec::<CompletionItem>::new();
                    for entry in fs::read_dir(abs_path).unwrap() {
                        let entry = entry.unwrap();
                        let path: PathBuf = entry.path();
                        let file_name = match path.file_name() {
                            Some(os_str) => os_str.to_str(),
                            None => continue
                        };

                        if file_name.is_none() {
                            continue;
                        }
                        let file_name = file_name.unwrap();

                        if path.is_dir() {
                            completion_items.push(CompletionItem {
                                label: file_name.to_string(),
                                kind: Some(CompletionItemKind::FOLDER),
                                ..CompletionItem::default()
                            });
                        }

                        if path.is_file() {
                            completion_items.push(CompletionItem {
                                label: file_name.to_string(),
                                kind: Some(CompletionItemKind::FILE),
                                ..CompletionItem::default()
                            });
                        }
                    }

                    if !completion_items.is_empty() {
                        return Some(CompletionList {
                            is_incomplete: false,
                            items: completion_items
                        });
                    }
                }
            }
        }
    }
    None
}